package lsedit;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;

public class MapBox extends JComponent implements Scrollable
{
	protected static final int GAP = 5; 

	protected static final Font textFont = new Font("Helvetica", Font.PLAIN, 10); 
	protected static final int TY_CLEAR	 = 0;
	protected static final int TY_RAISED = 1;
	protected static final int TY_SUNK	 = 2;

	class Map extends JComponent implements MouseListener
	{
		protected int					m_style;
		protected LandscapeViewerCore	m_ls;
		protected JLabel				m_label;
		protected EntityInstance		m_entity;

		public Map(LandscapeViewerCore ls, EntityInstance entity)
		{
			setLayout(null);
			
			m_style  = MapBox.TY_CLEAR;
			m_ls     = ls;

			m_label = new JLabel(entity.getLabel(), JLabel.CENTER);
			m_label.setForeground(Color.black);
			m_label.setFont(MapBox.textFont);
			m_label.setLocation(0,0);
			add(m_label);

			m_entity = entity;
			setToolTipText(entity.getLabel());
			addMouseListener(this);
		}
		
		public void setBounds(int x, int y, int width, int height)
		{
			super.setBounds(x,y,width,height);
			m_label.setSize(width, 20);
		}
		
		public void paintComponent(Graphics g)
		{
			EntityInstance	parent;
			int	x, y, width, height, lheight;

			x       = MapBox.GAP*4;
			y       = 0;
			width   = getWidth();
			height  = getHeight();
			lheight = m_label.getHeight();

			if (m_style != MapBox.TY_CLEAR) {
				g.setColor(Diagram.boxColour);
				g.draw3DRect(0, 0, width-2, height-2, m_style == MapBox.TY_RAISED);
			} 

/*			// For debugging
			g.setColor(Color.red);
			g.drawLine(0, 0, width-1, height-1);
			g.drawLine(0, height-1, width-1, 0);
*/
			g.setColor(Color.black);
			g.drawLine(0, height-1, width-1, height-1);

			parent = m_entity.getContainedBy();
			if (parent == null) {
				parent = m_entity;
			}
			parent.paintMap(g, (int) (width * 0.166), lheight+5, (int) (width * 0.66), height-lheight-10, m_entity, 0);
		}

		public void validate()
		{
			Diagram		diagram;
			int			label_height;
			int			width,	height;
			double		ratio;

			super.validate();

			diagram = m_ls.getDiagram();
			ratio   = ((double) diagram.getHeight()) / ((double) diagram.getWidth());
			width   = m_ls.m_tabWidth;
			width  -= MapBox.GAP*8;		// Workable width
			height  = (int) (width/2 * ratio);

			label_height = m_label.getHeight();
			m_label.setBounds(MapBox.GAP*4, 0, width, label_height);
		}

		public String toString()
		{
			return ("Map " + m_label.getText() + " " + getBounds());
		}

		// MouseListener interface

		public void mouseClicked(MouseEvent e)
		{
		}

		public void mouseEntered(MouseEvent e)
		{
			m_style = MapBox.TY_RAISED;
			repaint();
		}

		public void mouseExited(MouseEvent e)
		{
			m_style = MapBox.TY_CLEAR;
			repaint();
		}

		public void mousePressed(MouseEvent ev)
		{
			m_style = MapBox.TY_SUNK;
			repaint();
		}

		public void mouseReleased(MouseEvent ev)
		{
			m_style = MapBox.TY_CLEAR;
			m_ls.followLink(m_entity, false);
		}
	}

	protected LandscapeViewerCore   m_ls;
	protected JTabbedPane			m_tabbedPane;
	protected JScrollPane			m_scrollPane;
	protected String				m_title;
	protected EntityInstance		m_lastDrawRoot;

	public    static final String m_helpStr = "This box shows stylized versions of landscapes representing the path from the top of the containment hierarchy to the current landscape, with green boxes highlighting steps in the path.";

	// --------------
	// Public methods 
	// --------------

	public MapBox(LandscapeViewerCore ls, JTabbedPane tabbedPane) 
	{
		m_ls         = ls;
		m_tabbedPane = tabbedPane;
		m_title      = "Map";

		setLayout(null);
		setBackground(Diagram.boxColour);
		setToolTipText(m_helpStr);

		m_scrollPane = new JScrollPane();
//		m_scrollPane.setLayout(null);
		m_scrollPane.setBounds(tabbedPane.getBounds());
		setBounds(tabbedPane.getBounds());
		m_scrollPane.setViewportView(this);

		tabbedPane.addTab(m_title, null, m_scrollPane, m_helpStr);
	}

	public void activate() 
	{
		m_tabbedPane.setSelectedComponent(m_scrollPane);
	}

	public boolean isActive() 
	{
		Component active;

		if (!isVisible()) {
			return(false);
		}
		active = m_tabbedPane.getSelectedComponent();
		return(active != null && this == active);
	}

/*
	public void setBounds(int x, int y, int width, int height)
	{
		super.setBounds(x,y,width,height);
	}
*/
	
	public void validate()
	{
		Diagram			diagram;
		EntityInstance	drawRoot, e, parent;
		int				width, height, y;
		Map				map;

		diagram  = m_ls.getDiagram();
		if (diagram != null) {
			drawRoot = diagram.getDrawRoot();
			if (drawRoot != m_lastDrawRoot) {
				m_lastDrawRoot = drawRoot;
	
				removeAll();
				y      = 0;
				width  = m_tabbedPane.getWidth() - 20; /* Allow space for scroll bar */
				height = (int) ( ((double) width) * 0.66);
							
				for (e = diagram.getDrawRoot(); (parent = e.getContainedBy()) != null; e = parent) {
					if (e.isEnterable()) {
						y += height + GAP;
				}	}
		
				setBounds(0,0,width, y);
				width -= 2*GAP;
					
				for (e = diagram.getDrawRoot(); (parent = e.getContainedBy()) != null; e = parent) {
					if (e.isEnterable()) {
						y -= height + GAP;
						map = new Map(m_ls, e);
						map.setBounds(GAP, y, width, height);
						add(map);
		}	}	}	}
		repaint();
	}
	
	public Dimension getPreferredSize()
	{
		return(getSize());
	}
	
	public Dimension getMinimumSize()
	{
		return(getSize());
	}
	
	public Dimension getMaximumSize()
	{
		return(getSize());
	}


/*
	public void paintComponent(Graphics g)
	{
		int	width, height;

		width   = getWidth();
		height  = getHeight();
		
		// For debugging
		g.setColor(Color.green);
		g.drawLine(0, 0, width, height);
		g.drawLine(0, height, width, 0);
	}
*/
	// Scrollable interface
	// We need to implement this to stop iterative size changes..
	// The mapbox knows what size it wants to be and sets it on every validate
	// If we don't disable it the viewport tries to change the size to fit the window every time it validates

	public Dimension getPreferredScrollableViewportSize()
	{
		return(getSize());
	}
	 
	public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction)
	{
		return(50);
	}
	 
	public boolean getScrollableTracksViewportHeight() 
	{
		return(false);
	}

	public boolean getScrollableTracksViewportWidth() 
	{
		return(false);
	}

	public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction)
	{
		return(10);
	} 
}
